@@ -1,9 +1,8 @@
 #! /usr/bin/perl -w
 
-# Copyright (c) 2003-2016 Henk Penning, all rights reserved.
-# penning@uu.nl, http://www.staff.science.uu.nl/~penni101/
+# Copyright (c) 2003 Henk Penning, all rights reserved.
+# penning@cs.uu.nl, http://www.cs.uu.nl/staff/henkp.html
 # Version 1.1 was donated to the Apache Software Foundation 2003 Jan 28.
-#
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
 # to deal in the Software without restriction, including without limitation
@@ -29,16 +28,16 @@
 use strict ;
 
 our $PRG = 'mirmon' ;
-our $VER = "2.11" ;
+our $VER = "2.3" ;
 
 our $DEF_TIMEOUT = 300 ;
 our $HIST        = 14 ;
 our $TIM_PAT     = '^(\d+)([smhd])$' ;
-our %APA_TYPES   = () ; $APA_TYPES { $_ } ++ for qw(backup ftp http rsync) ;
-our %GET_OPTS    = () ; $GET_OPTS  { $_ } ++ for qw(all update url) ;
+our %APA_TYPES   = () ; for ( qw(backup ftp http) ) { $APA_TYPES { $_ } ++ ; }
+our %GET_OPTS    = () ; for ( qw(all update) )      { $GET_OPTS  { $_ } ++ ; }
 our $HIST_DELTA  = 24 * 60 * 60 ;
-our $APRX_DELTA  = 300 ;
-our $HOME        = 'http://www.staff.science.uu.nl/~penni101/mirmon/' ;
+our $APRX_DELTA  = 60 ;
+our $HOME        = 'http://www.cs.uu.nl/people/henkp/mirmon/' ;
 
 package Base ; #####################################################
 
@@ -212,7 +211,7 @@ sub find_config
   { my $self = shift ;
     my $arg = shift ;
     my @LIST = $arg ? ( $arg ) : Mirmon -> config_list ;
-    for my $conf ( @LIST ) { return $conf if -r $conf and ! -d $conf ; }
+    for my $conf ( @LIST ) { return $conf if -f $conf ; }
     die sprintf "can't find a config file :\n  %s\n" , join "\n  ", @LIST ;
   }
 
@@ -226,7 +225,6 @@ sub get_config
 sub get_state
   { my $self = shift ;
     my $conf = $self -> conf ;
-    my $name = $conf -> project_name ;
     my $state = $conf -> state ;
     my $res = {} ;
     open STATE, $state or die "can't open $state ($!)" ;
@@ -240,7 +238,6 @@ sub get_state
     my $mlist = $conf -> mirror_list ;
     my $style = $conf -> list_style ;
     my %in_list = () ;
-    my $changes = '' ;
     open MLIST, $mlist or die "can't open $mlist ($!)" ;
     for my $line ( <MLIST> )
       { chop $line ;
@@ -267,7 +264,7 @@ sub get_state
         $in_list { $url } ++ ;
 
         unless ( exists $res -> { $url } )
-          { $changes .= sprintf "added %s\n", $url unless Mirmon::quiet ;
+          { printf "*** added to list %s\n", $url unless Mirmon::quiet ;
             $res -> { $url } = Mirmon::Mirror -> init ( $self, $url ) ;
           }
         my $mirror = $res -> { $url } ;
@@ -279,12 +276,10 @@ sub get_state
     for my $url ( sort keys %$res )
       { # printf "%s\n", $res -> { $url } -> state ;
         unless ( exists $in_list { $url } )
-          { $changes .= sprintf "removed %s\n", $url unless Mirmon::quiet ;
+          { printf "*** removed from list %s\n", $url unless Mirmon::quiet ;
             delete $res -> { $url } ;
           }
       }
-    printf "changes in mirror-list for '%s':\n%s", $name, $changes
-      if $changes ;
     $self -> state ( $res ) ;
   }
 
@@ -314,175 +309,73 @@ sub get_regions
       { chop ;
         next if /^#/ ;
         my ( $code, $dash, $reg ) = split ' ', $_, 3 ;
-        $self -> { regions } { lc $code } = $reg ;
+        $self -> { regions } { lc $code } = lc $reg ;
       }
     close REGS ;
   }
 
-sub _cmp_ccs
-  { my $ccs = shift ;
-    my $x  = shift ;
-    my $y  = shift ;
-    my $xx = $ccs -> { $x } ;
-    my $yy = $ccs -> { $y } ;
-    if ( ! defined $xx and ! defined $yy )
-      { $x cmp $y ; }
-    elsif ( ! defined $xx )
-      { -1 ; }
-    elsif ( ! defined $yy )
-      { +1 ; }
-    else
-      { $xx cmp $yy ; }
-  }
-
-sub _pr_round
-  { my $x = shift ;
-    my $i = int $x ;
-    my $f = $x - $i ;
-    $i + ( rand 1 < $f ? 1 : 0 ) ;
-  }
-
-sub _diag_qs
-  { my $qs = shift ;
-    join ', ', map { sprintf "%s %s" , $_, scalar @{ $qs -> { $_ } } ; }
-      sort keys %$qs ;
-  }
-
-sub _rpick
-  { my $row = shift ;
-    die "_rpick : row empty" unless @$row ;
-    my $idx = int rand @$row ;
-    my $res = $row -> [ $idx ] ;
-    $row -> [ $idx ] = $row -> [ $#{$row} ] ;
-    pop @$row ;
-    $res ;
-  }
-
-sub _buck_split
-  { my $que = shift ;
-    my $tmp = [] ;
-    for my $mirr ( @$que )
-      { my $lp = $mirr -> last_probe ;
-        my $hr = int ( ( $^T - $lp ) / 60 / 60 + 0.5 ) ;
-        push @{ $tmp -> [ $hr ] }, $mirr ;
-      }
-    [ grep defined $_, @$tmp ] ;
-  }
-
-sub _buck_join
-  { my $bucks = shift ;
-    my $res = [] ;
-    push @$res, @$_ for @$bucks ;
-    $res ;
-  }
-
-sub _buck_pick
-  { my $bucks = shift ;
-    die "buck_pick : bucks empty" unless @$bucks ;
-    my $buck = ( sort { @$b <=> @$a } @$bucks ) [ 0 ] ;
-    _rpick $buck ;
-  }
-
-sub _randomize
-  { my $ques = shift ;
-    my $poll = shift ;
-    my $hrs  = int ( $poll / 60 / 60 + 0.5 ) ;
-
-    my $diag1 = _diag_qs $ques ;
-
-    my $todos = $ques -> { todo } ;
-    my $dones = $ques -> { done } ;
-    my $cnt   = @$todos + @$dones ;
-    my $avg   = $hrs ? $cnt / $hrs : 0 ;
-    my $iavg  = _pr_round $avg ;
-    my $pick  = 0 ;
-    my $bucks = _buck_split $dones ;
-
-    while ( @$todos < $iavg and $pick < @$dones )
-      { push @$todos, _buck_pick $bucks ;
-        $pick ++ ;
-      }
-
-    $ques -> { done } = _buck_join $bucks ;
-
-    sprintf ''
-      . "  hrs %s, %s\n"
-      . "  avg %.2f -> %d , picked %d ; queued %s\n"
-      . "  hrs %s, %s\n"
-      , $hrs, $diag1
-      , $avg, $iavg, $pick, scalar @$todos
-      , $hrs, _diag_qs ( $ques )
-      ;
-  }
-
 sub get_dates
   { my $self  = shift ;
     my $get   = shift ;
-    my $URL   = shift ;
     my $state = $self -> state ;
-    my $conf  = $self -> conf ;
-    my $CMD   = $conf -> probe ;
-    my $PAR   = $conf -> max_probes ;
-    my %m4h   = () ;
-    my @QUE   = () ;
-    my $GET   = IO::Select -> new () ;
-    my $ques  = {} ;
-    for my $col ( qw(new red grn xtr) )
-      { $ques -> { $col } { $_ } = [] for qw(done todo) ; }
-    my $max_poll = s4tim $conf -> max_poll ;
-    my $min_poll = s4tim $conf -> min_poll ;
+    my $conf = $self -> conf ;
+    my $CMD  = $conf -> probe ;
+    my $PAR  = $conf -> max_probes ;
+    my %m4h  = () ;
+    my @QUE  = () ;
+    my @NOQ  = () ;
+    my $GET = IO::Select -> new () ;
 
-    if ( Mirmon::verbose ) { printf "mirrors %d\n", scalar keys %$state ; }
+    my $cnt = 0 ;
+    my $nok = 0 ;
 
-    if ( $get eq 'all' )
-      { @QUE = sort { $a -> url cmp $b -> url } values %$state ; }
-    elsif ( $get eq 'url' )
-      { @QUE = ( $state -> { $URL } ) ; }
-    elsif ( $get eq 'update' )
-      { my $maxp = $^T - $max_poll ;
-        my $minp = $^T - $min_poll ;
-
-if ( Mirmon::verbose )
-  { printf "max_poll %s\n", scalar localtime $maxp ;
-    printf "min_poll %s\n", scalar localtime $minp ;
-  }
-        for my $url ( sort keys %$state )
-          { my $mirror = $state -> { $url } ;
-            my $stat = $mirror -> last_status ;
+    for my $url ( sort keys %$state )
+      { my $mirror = $state -> { $url } ;
+        $cnt ++ if $mirror -> last_status eq 'ok' ;
+        if ( $get eq 'all' or $mirror -> last_probe eq 'undef' )
+          { push @QUE, $mirror ; }
+        elsif ( $get eq 'update' )
+          { my $stat = $mirror -> last_status ;
             my $vrfy = $mirror -> last_ok_probe ;
             my $lprb = $mirror -> last_probe ;
-            my $col ;
-            my $que ;
-            if ( $stat eq 'undef' ) # never probed ; new mirror ; todo
-              { $col = 'new' ; $que = 'todo' ; }
-            elsif ( $conf -> get_xtr ( $mirror -> region ) )
-              { $col = 'xtr' ; $que = 'todo' ; }
-            else
-              { my $poll = $stat eq 'ok' ? $maxp : $minp ;
-                $col     = $stat eq 'ok' ? 'grn' : 'red' ;
-                $que     = ( aprx_le $lprb, $poll ) ? 'todo' : 'done' ;
+            if ( aprx_le $lprb, $^T - s4tim $conf -> min_poll )
+              { if ( $stat ne 'ok' )
+                  { push @QUE, $mirror ; $nok ++ ; }
+                elsif ( aprx_le $vrfy, $^T - s4tim $conf -> max_poll )
+                  { push @QUE, $mirror ; }
+                else
+                  { push @NOQ, $mirror ; }
               }
-            push @{ $ques -> { $col } { $que } }, $mirror ;
           }
-
-        if ( $conf -> randomize )
-          { my $msg = "randomize green\n" ;
-            $msg .= _randomize $ques -> { grn }, $max_poll ;
-            $msg .= "randomize red\n" ;
-            $msg .= _randomize $ques -> { red }, $min_poll ;
-            print $msg if Mirmon::verbose ;
-          }
-        @QUE =
-          ( @{ $ques -> { new } { todo } }
-          , @{ $ques -> { red } { todo } }
-          , @{ $ques -> { grn } { todo } }
-          , @{ $ques -> { xtr } { todo } }
-          ) ;
+        else
+          { die "unknown opt_get '$get'" ; }
       }
-    else
-      { die "unknown opt_get '$get'" ; }
 
-    if ( Mirmon::verbose ) { printf "queued %d\n\n", scalar @QUE ; }
+    if ( Mirmon::verbose )
+      { my $que = scalar @QUE ; my $noq = scalar @NOQ ;
+        printf "ok mirrors %d, queued %d, not queued %d, ok %d, nok %d\n"
+          , $cnt, $que, $noq, $que - $nok, $nok
+      }
+
+    if ( $conf -> randomize )
+      { my $hrs  = int ( ( s4tim $conf -> max_poll ) / 60 / 60 + 0.5 ) ;
+        my $avg  = int ( $cnt / $hrs + 0.5 ) ;
+        my $prc  = ( scalar keys %$state ) / 50 ;
+        my $flr  = int $prc ;
+        my $extras = $flr + ( rand 1 < ( $prc - $flr ) ) ;
+        my $picked = 0 ;
+
+        while ( @QUE < $avg + $nok and @NOQ and $picked < $extras )
+          { my $idx = int rand @NOQ ;
+            push @QUE, $NOQ [ $idx ] ;
+            $NOQ [ $idx ] = $NOQ [ $#NOQ ] ;
+            pop @NOQ ;
+            $picked ++ ;
+          }
+
+        printf "avg mirrors/hr %d, max extras %d, picked %d ; queued %s\n"
+          , $avg, $extras, $picked, scalar @QUE if Mirmon::verbose ;
+      }
 
     while ( @QUE )
       { my $started = 0 ;
@@ -665,22 +558,17 @@ sub age_avg
     for my $age ( @tab ) { $tot += $age ; }
     my $mean = $tot / $cnt ;
 
-# @tab   0  1  2  3  4  5  6  7
-# $#tab -1  0  1  2  3  4  5  6
-# mid    U  0  0  1  1  2  2  3
-# res    U  0 0+1 1 1+2 2 2+3 3
-
     my $median ;
     if ( $cnt == 1 )
       { $median = $tab [ 0 ] ; }
-    elsif ( $cnt % 2 ) # cnt is odd ; $#tab is even
-      { my $mid = int ( $#tab / 2 + 0.5 ) ;
-        $median = $tab [ $mid ] ;
-      }
-    else # cnt is even ; $#tab is odd
+    elsif ( $cnt % 2 )
       { my $mid = int ( $#tab / 2 ) ;
         $median = ( $tab [ $mid ] + $tab [ $mid + 1 ] ) / 2 ;
       }
+    else
+      { my $mid = int ( $#tab / 2 ) ;
+        $median = $tab [ $mid ] ;
+      }
 
     if ( @tab < 2 )
       { return $mean, $median, undef ; }
@@ -983,8 +871,7 @@ sub gen_histogram
       { $res .= sprintf "<BR>each %s unit represents %s mirror sites.\n"
           , $units, sprintf ( "%.1f", $max / $H ) ;
       }
-    return H2 ( NAM 'age-histogram', 'age histogram' )
-      . BQ $res ;
+    return H2 ( 'age histogram' ) . BQ $res ;
   }
 
 sub gen_page
@@ -1133,11 +1020,10 @@ $histo_top
 HEAD
 
     for my $reg
-      ( sort { _cmp_ccs $CCS, $a, $b } keys %tab )
-#      { ( $CCS -> { $a } ? lc ( $CCS -> { $a } ) : $a )
-#    cmp ( $CCS -> { $b } ? lc ( $CCS -> { $b } ) : $b )
-#      } keys %tab
-#     )
+      ( sort
+       { ( $CCS -> { $a } || $a ) cmp ( $CCS -> { $b } || $b ) ; }
+       keys %tab
+      )
       { my $mirrors = $tab { $reg } ;
 
         my $ccs = exists $CCS -> { $reg } ? $CCS -> { $reg } : $reg ;
@@ -1152,8 +1038,8 @@ HEAD
         for my $mirror ( sort { $a -> cmp ( $b ) } @$mirrors )
           { print "<TR>\n" ;
             printf "  <TD ALIGN=RIGHT>%s&nbsp;&nbsp;%s</TD>\n  <TD>%s</TD>\n"
-              , $mirror -> url_site
-              , $mirror -> url_home
+              , $mirror -> site_url
+              , $mirror -> home_url
               , $mirror -> type
               ;
 
@@ -1232,7 +1118,6 @@ our %CNF_defaults =
   , htm_top      => ''
   , htm_foot     => ''
   , htm_head     => ''
-  , always_get   => ''
   ) ;
 
 our @REQ_KEYS =
@@ -1247,12 +1132,6 @@ my @PUT_HGRAM  = qw(top bottom nowhere) ;
 
 eval Base -> mk_methods ( keys %CNF_KEYS, qw(root site_url) ) ;
 
-sub get_xtr
-  { my $self = shift ;
-    my $reg  = shift ;
-    scalar grep { $_ eq $reg } split ' ', $self -> always_get ;
-  }
-
 sub new
   { my $self = shift ;
     my $FILE = shift ;
@@ -1414,18 +1293,15 @@ sub state_history_hist
 sub _parse
   { my $self = shift ;
     my $url  = $self -> url ;
-    my ( $type, $site, $home, $path ) ;
+    my ( $type, $site, $home ) ;
     if ( $url =~ m!^(ftp|https?|rsync)://([^/:]+)(:\d+)?/! )
-      { $type = $1 ; $site = $2 ; $home = $& ; $path = $' ; }
-    else
-      { warn "can't parse url ($url)" ; }
-    return $type, $site, $home, $path ;
+      { $type = $1 ; $site = $2 ; $home = $& ; }
+    return $type, $site, $home ;
   }
 
 sub type { my $self = shift ; ( $self -> _parse ) [ 0 ] ; }
 sub site { my $self = shift ; ( $self -> _parse ) [ 1 ] ; }
 sub home { my $self = shift ; ( $self -> _parse ) [ 2 ] ; }
-sub path { my $self = shift ; ( $self -> _parse ) [ 3 ] ; }
 
 sub age_in_days
   { my $self = shift ;
@@ -1553,7 +1429,7 @@ sub finish_probe
         if ( $res !~ /^\d+$/ )
           { $res =~ s/ /_/g ;
             $res = Base::htmlquote $res ;
-            $res = substr ( $res, 0, 15 ) . '..' if length $res > 15 ;
+            $res = substr ( $time, 0, 15 ) . '..' if length $res > 15 ;
             $stat = "'$res'" ;
           }
         else
@@ -1587,26 +1463,8 @@ sub _url
     $hrf =~ /^rsync/ ? $txt : URL $hrf, $txt ;
   }
 
-sub url_site
-  { my $self = shift ;
-    my $type = $self -> type ;
-    if ( $type eq 'rsync' )
-      { my $path = $self -> path ;
-        chop $path if $path =~ m!/$! ;
-        sprintf '%s::%s', $self -> site , $path ;
-      }
-    else
-      { URL $self -> url , $self -> site ; }
-  }
-
-sub url_home
-  { my $self = shift ;
-    my $type = $self -> type ;
-    if ( $type eq 'rsync' )
-      { '@' ; }
-    else
-      { URL $self -> home, '@' ; }
-  }
+sub site_url { my $self = shift ; _url $self -> url , $self -> site ; }
+sub home_url { my $self = shift ; _url $self -> home, '@' ; }
 
 =pod
 
@@ -1679,10 +1537,9 @@ Returns a hashref C<< { country_code =E<gt> country_name, ... } >>.
 
 Returns the list of default locations for config files.
 
-=item B<get_dates ( $get [, $URL] )>
+=item B<get_dates ( $get )>
 
-Probes all mirrors if $get is C<all> ; or a subset if $get is C<update> ;
-or only I<$URL> if $get is C<url>.
+Probes all mirrors if $get is C<all> ; or a subset if $get is C<update>.
 
 =back
 
@@ -1730,7 +1587,7 @@ The url as given in the mirror list.
 
 =item B<age>
 
-The mirror's timestamp found by the last successful probe,
+The mirror's timestamp found by the last succesful probe,
 or 'undef' if no probe was ever successful.
 
 =item B<last_status>
@@ -1739,7 +1596,7 @@ The status of the last probe, or 'undef' if the mirror was never probed.
 
 =item B<last_ok_probe>
 
-The timestamp of the last successful probe or 'undef'
+The timestamp of the last succesful probe or 'undef'
 if the mirror was never successfully probed.
 
 =item B<probe_history>
@@ -1827,9 +1684,7 @@ and updates the state of the mirror.
 
 =begin html
 
-<p>
-<a href="mirmon.html">mirmon(1)</a>
-</p>
+<A HREF="mirmon.html">mirmon(1)</A>
 
 =end html
 
@@ -1843,33 +1698,30 @@ mirmon(1)
 
 =begin html
 
-  <p>
-  &copy; 2003-2016
-  <a href="http://www.staff.science.uu.nl/~penni101/">Henk P. Penning</a>,
-  <a href="http://www.uu.nl/faculty/science/EN/">Faculty of Science</a>,
-  <a href="http://www.uu.nl/">Utrecht University</a>
-  <br />
-  mirmon-2.11 - Sat Jul 23 09:12:31 2016 ; henkp ;
-  <a href="http://validator.w3.org/check?uri=referer">verify html</a>
-  </p>
+  &copy; 2003-2010
+  <A HREF="http://people.cs.uu.nl/henkp/">Henk P. Penning</A>,
+  <A HREF="http://www.cs.uu.nl/">Computer Science Department</A>,
+  <A HREF="http://www.uu.nl/">Utrecht University</A>
+  <BR>
+  mirmon-2.3 - Wed Mar 17 09:29:11 2010 ; henkp
 
 =end html
 
 =begin man
 
-  (c) 2003-2016 Henk P. Penning
-  Faculty of Science, Utrecht University
-  http://www.staff.science.uu.nl/~penni101/ -- penning@uu.nl
-  mirmon-2.11 - Sat Jul 23 09:12:31 2016 ; henkp
+  (c) 2003-2010 Henk P. Penning
+  Computer Science Department, Utrecht University
+  http://people.cs.uu.nl/henkp/ -- penning@cs.uu.nl
+  mirmon-2.3 - Wed Mar 17 09:29:11 2010 ; henkp
 
 =end man
 
 =begin text
 
-  (c) 2003-2016 Henk P. Penning
-  Faculty of Science, Utrecht University
-  http://www.staff.science.uu.nl/~penni101/ -- penning@uu.nl
-  mirmon-2.11 - Sat Jul 23 09:12:31 2016 ; henkp
+  (c) 2003-2010 Henk P. Penning
+  Computer Science Department, Utrecht University
+  http://people.cs.uu.nl/henkp/ -- penning@cs.uu.nl
+  mirmon-2.3 - Wed Mar 17 09:29:11 2010 ; henkp
 
 =end text
 
